import java.util.LinkedList;
import java.util.NoSuchElementException;

public class ListMap<K, V> {
    private LinkedList<Association<K, V>> items;

    public ListMap() {
        items = new LinkedList<>();
    }

    private Association<K, V> findKey(K key) {
        for (Association<K, V> a : items) {
            if (a.getKey().equals(key)) {
                return a;
            }
        }
        return null;
    }

    public V put(K key, V value) {
        // find existing entry for key, if it exists
        Association<K, V> a = findKey(key);
        if (a != null) {
            items.remove(a);
            items.add(new Association<K, V>(key, value));
            return a.getValue();
        }
        // no existing key, add new one and return null
        items.add(new Association<K, V>(key, value));
        return null;
    }

    public boolean contains(K key) {
        return findKey(key) != null; // return null if key isn't there
    }

    public V get(K key) {
        Association<K, V> a = findKey(key);
        if (a == null) {
            throw new NoSuchElementException();
        }
        return a.getValue();
    }

    public V remove(K key) {
        Association<K, V> a = findKey(key);
        if (a == null) {
            throw new NoSuchElementException();
        }
        items.remove(a);
        return a.getValue();
    }

    public String toString() {
        StringBuilder ret = new StringBuilder();
        for (Association<K, V> a : items) {
            ret.append(a.getKey() + " -> " + a.getValue() + "\n");
        }
        // return string with the final \n trimmed off
        return ret.toString().substring(0, ret.length() - 1);
    }

    public static void main(String[] args) {
        // a little bit of testing
        ListMap<String, Integer> map = new ListMap<>();
        map.put("a", 5);
        map.put("c", 2);
        map.put("a", 4);
        System.out.println(map.remove("c"));
        System.out.println(map);
        map.put("t", 8);
        System.out.println(map.get("a") + map.get("t"));
    }
}
